@@ -1,12 +1,9 @@
-#!/usr/bin/env python3
-
 """
-Class to parse command line arguments.
+Class to parse command line arguments for all commands.
 """
 
 import argparse
 import sys
-from datetime import datetime
 from typing import List, Optional
 
 from yd_commands.__init__ import __version__
@@ -20,6 +17,20 @@ def docs():
     )
 
 
+def allow_missing_attribute(func):
+    """
+    Wrapper function to return None if an option isn't enabled.
+    """
+
+    def wrapper(*args, **kwargs):
+        try:
+            return func(*args, **kwargs)
+        except AttributeError:
+            return None
+
+    return wrapper
+
+
 class CLIParser:
     def __init__(self, description: Optional[str] = None):
         """
@@ -42,7 +53,7 @@ class CLIParser:
             type=str,
             help=(
                 "configuration file in TOML format; "
-                "default is 'config.toml' in the current directory"
+                "the default to use is 'config.toml' in the current directory"
             ),
             metavar="<config_file.toml>",
         )
@@ -67,7 +78,12 @@ class CLIParser:
             "-n",
             type=str,
             required=False,
-            help="the namespace to use when creating and identifying entities",
+            nargs="?",
+            const="",
+            help=(
+                "the namespace to use when creating and identifying entities;"
+                " this is set to '' if the option is provided without a value"
+            ),
             metavar="<namespace>",
         )
         parser.add_argument(
@@ -76,7 +92,13 @@ class CLIParser:
             "--prefix",
             type=str,
             required=False,
-            help="the tag/prefix for naming or selecting entities",
+            nargs="?",
+            const="",
+            help=(
+                "the tag to use when naming, tagging, or selecting entities,"
+                " or the prefix (directory) when used with the object store;"
+                " this is set to '' if the option is provided without a value"
+            ),
             metavar="<tag>",
         )
         parser.add_argument(
@@ -93,7 +115,10 @@ class CLIParser:
             type=str,
             required=False,
             action="append",
-            help="user-defined variable substitutions; can be supplied multiple times",
+            help=(
+                "user-defined variable substitution; the option can be supplied"
+                " multiple times, one per variable"
+            ),
             metavar="<var1=v1>",
         )
         parser.add_argument(
@@ -107,7 +132,7 @@ class CLIParser:
             "--debug",
             action="store_true",
             required=False,
-            help="print a stack trace (etc.) on error",
+            help="display the Python stack trace on error",
         )
         parser.add_argument(
             "--pac",
@@ -115,6 +140,12 @@ class CLIParser:
             required=False,
             help="enable PAC (proxy auto-configuration) support",
         )
+        parser.add_argument(
+            "--no-format",
+            action="store_true",
+            required=False,
+            help="disable colouring and text wrapping in command output",
+        )
 
         # Module-specific argument sets
 
@@ -147,7 +178,7 @@ class CLIParser:
                 "-X",
                 type=str,
                 required=False,
-                help="the executable to use",
+                help="the 'executable' to use in the case of certain task types",
                 metavar="<executable>",
             )
             parser.add_argument(
@@ -163,7 +194,7 @@ class CLIParser:
                 "-C",
                 type=int,
                 required=False,
-                help="the number of times to submit the task",
+                help="the number of times to submit the Task",
                 metavar="<task_count>",
             )
             parser.add_argument(
@@ -171,15 +202,21 @@ class CLIParser:
                 "-b",
                 type=int,
                 required=False,
-                help="the batch size for task submission",
+                help="the batch size for task submission; must be between 1 and 10,000",
                 metavar="<batch_size>",
             )
             parser.add_argument(
                 "--pause-between-batches",
                 "-P",
-                action="store_true",
+                nargs="?",
+                type=int,
+                const=0,
                 required=False,
-                help="pause for user input between batches (for debugging)",
+                metavar="<interval_between_batches_in_seconds>",
+                help=(
+                    "pause for user input between task batch submissions; if no"
+                    " pause interval is provided, user input is required to advance"
+                ),
             )
             parser.add_argument(
                 "--csv-file",
@@ -187,7 +224,7 @@ class CLIParser:
                 type=str,
                 required=False,
                 action="append",
-                help="the CSV file(s) from which to read task data",
+                help="the CSV file(s) from which to read Task data",
                 metavar="<data.csv>",
             )
             parser.add_argument(
@@ -196,10 +233,17 @@ class CLIParser:
                 action="store_true",
                 required=False,
                 help=(
-                    "process CSV variable substitutions only and output intermediate"
-                    " JSON"
+                    "process CSV variable substitutions only and output the"
+                    " intermediate JSON Work Requirement specification"
                 ),
             )
+            parser.add_argument(
+                "--hold",
+                "-H",
+                action="store_true",
+                required=False,
+                help="set the work requirement status to 'HELD' on submission",
+            )
 
         if any(module in sys.argv[0] for module in ["provision", "instantiate"]):
             parser.add_argument(
@@ -224,7 +268,22 @@ class CLIParser:
                 "-f",
                 action="store_true",
                 required=False,
-                help="when using --abort, poll until all Tasks have been aborted",
+                help="follow progress after cancelling the work requirement(s)",
+            )
+
+        if any(
+            module in sys.argv[0]
+            for module in [
+                "start",
+                "hold",
+            ]
+        ):
+            parser.add_argument(
+                "--follow",
+                "-f",
+                action="store_true",
+                required=False,
+                help="follow work requirement events after applying action",
             )
 
         if any(
@@ -235,6 +294,8 @@ class CLIParser:
                 "download",
                 "shutdown",
                 "terminate",
+                "start",
+                "hold",
             ]
         ):
             parser.add_argument(
@@ -254,6 +315,7 @@ class CLIParser:
                 "shutdown",
                 "terminate",
                 "resize",
+                "cloudwizard",
             ]
         ):
             parser.add_argument(
@@ -285,6 +347,7 @@ class CLIParser:
                 "--directory",
                 "-d",
                 type=str,
+                default="",
                 required=False,
                 help=(
                     "the directory to use for downloaded objects (defaults to the"
@@ -298,19 +361,25 @@ class CLIParser:
                 action="store_true",
                 required=False,
                 help=(
-                    "flatten download paths (objects with the same name will be"
-                    " overwritten)"
+                    "flatten download paths (warning: objects with the same filenames"
+                    " will be overwritten)"
                 ),
             )
 
         if "list" in sys.argv[0]:
+            parser.add_argument(
+                "--reverse",
+                action="store_true",
+                required=False,
+                help="list items in reverse-sorted name order",
+            )
             parser.add_argument(
                 "--object-paths",
                 "--objects",
                 "-o",
                 action="store_true",
                 required=False,
-                help="list YellowDog Object Store object paths",
+                help="list object store object paths",
             )
             parser.add_argument(
                 "--all",
@@ -319,7 +388,7 @@ class CLIParser:
                 required=False,
                 help=(
                     "when used with '--objects', list all objects, not just the top"
-                    " level structure"
+                    " level prefixes"
                 ),
             )
             parser.add_argument(
@@ -327,86 +396,87 @@ class CLIParser:
                 "-d",
                 action="store_true",
                 required=False,
-                help=(
-                    "select items from the list of matches and show their full details"
-                ),
+                help="show the full details of (interactively) selected objects",
             )
             parser.add_argument(
                 "--work-requirements",
                 "-w",
                 action="store_true",
                 required=False,
-                help="list Work Requirements",
+                help="list work requirements",
             )
             parser.add_argument(
                 "--task-groups",
                 "-g",
                 action="store_true",
                 required=False,
-                help="list Task Groups in selected Work Requirements",
+                help="list task groups in selected work requirements",
             )
             parser.add_argument(
                 "--tasks",
                 "-T",
                 action="store_true",
                 required=False,
-                help="list Tasks in selected Work Requirements",
+                help="list tasks in selected work requirements / task groups",
             )
             parser.add_argument(
                 "--worker-pools",
                 "-p",
                 action="store_true",
                 required=False,
-                help="list Worker Pools",
+                help="list worker pools",
             )
             parser.add_argument(
                 "--compute-requirements",
                 "-r",
                 action="store_true",
                 required=False,
-                help="list Compute Requirements",
+                help="list compute requirements",
             )
             parser.add_argument(
-                "--live-only",
+                "--active-only",
                 "-l",
                 action="store_true",
                 required=False,
-                help="list only active Compute Requirements/Worker Pools",
+                help=(
+                    "list only active compute requirements / worker pools / work"
+                    " requirements"
+                ),
             )
             parser.add_argument(
                 "--compute-templates",
                 "-C",
                 action="store_true",
                 required=False,
-                help="list Compute Requirement Templates",
+                help="list compute requirement templates",
             )
             parser.add_argument(
                 "--source-templates",
                 "-S",
                 action="store_true",
                 required=False,
-                help="list Compute Source Templates",
+                help="list compute source templates",
             )
             parser.add_argument(
                 "--keyrings",
                 "-K",
                 action="store_true",
                 required=False,
-                help="list Keyrings",
+                help="list keyrings",
             )
             parser.add_argument(
                 "--image-families",
                 "-I",
                 action="store_true",
                 required=False,
-                help="list Machine Image Families",
+                help="list machine image families",
             )
             parser.add_argument(
-                "--namespaces",
+                "--namespace-storage-configurations",
                 "-N",
                 action="store_true",
                 required=False,
-                help="list Namespace Storage Configurations",
+                help="list namespace storage configurations",
             )
             parser.add_argument(
                 "--instances",
@@ -415,21 +485,13 @@ class CLIParser:
                 help="list compute instances",
             )
 
-        if "upload" in sys.argv[0]:
-            parser.add_argument(
-                "--content-path",
-                "-C",
-                type=str,
-                required=False,
-                help="the directory where the items to be uploaded are found",
-                metavar="<directory>",
-            )
+        if any(module in sys.argv[0] for module in ["upload"]):
             parser.add_argument(
                 "filenames",
                 metavar="<filename>",
                 type=str,
                 nargs="+",
-                help="files and/or directories to upload to the Object Store",
+                help="files and/or directories to upload to the object store",
             )
             parser.add_argument(
                 "--flatten-upload-paths",
@@ -485,7 +547,7 @@ class CLIParser:
                 "-r",
                 action="store_true",
                 required=False,
-                help="report on a Dynamic Template test run",
+                help="report on a dynamic template test run",
             )
 
         if "admin" in sys.argv[0]:
@@ -515,7 +577,7 @@ class CLIParser:
                 metavar="<worker-pool-or-compute-requirement-name-or-ID>",
                 type=str,
                 help=(
-                    "the name or YellowDog ID of the Worker Pool or Compute Requirement"
+                    "the name or YellowDog ID of the worker pool or compute requirement"
                     " to resize"
                 ),
             )
@@ -523,44 +585,87 @@ class CLIParser:
                 "worker_pool_size",
                 metavar="<new-node/instance-count>",
                 type=int,
-                help="the desired number of (total) nodes in the Worker Pool",
+                help="the desired number of (total) nodes in the worker pool",
             )
             parser.add_argument(
                 "--compute-requirement",
                 "-C",
                 action="store_true",
                 required=False,
-                help="resize a Compute Requirement instead of a Worker Pool",
+                help="resize a compute requirement instead of a worker pool",
             )
 
         if "shutdown" in sys.argv[0]:
             parser.add_argument(
-                "worker_pool",
-                nargs="?",
+                "worker_pool_list",
+                nargs="*",
                 default="",
                 metavar="<worker-pool-name-or-ID>",
                 type=str,
-                help="the name or YellowDog ID of the Worker Pool",
+                help="the name(s) or YellowDog ID(s) of the worker pool(s)",
+            )
+            parser.add_argument(
+                "--follow",
+                "-f",
+                action="store_true",
+                required=False,
+                help="follow worker pool shutdown to completion",
             )
 
         if "terminate" in sys.argv[0]:
             parser.add_argument(
-                "compute_requirement",
-                nargs="?",
+                "compute_reqs",
+                nargs="*",
                 default="",
                 metavar="<compute-requirement-name-or-ID>",
                 type=str,
-                help="the name or YellowDog ID of the Compute Requirement",
+                help="the name(s) or YellowDog ID(s) of the compute requirement(s)",
+            )
+            parser.add_argument(
+                "--follow",
+                "-f",
+                action="store_true",
+                required=False,
+                help="follow termination to completion",
             )
 
         if "cancel" in sys.argv[0]:
             parser.add_argument(
-                "work_requirement",
-                nargs="?",
+                "work_requirements",
+                nargs="*",
                 default="",
                 metavar="<work-requirement-name-or-ID>",
                 type=str,
-                help="the name or YellowDog ID of the Work Requirement",
+                help=(
+                    "the name(s) or YellowDog ID(s) of the work requirement(s) to be"
+                    " cancelled"
+                ),
+            )
+
+        if "start" in sys.argv[0]:
+            parser.add_argument(
+                "work_requirements",
+                nargs="*",
+                default="",
+                metavar="<work-requirement-name-or-ID>",
+                type=str,
+                help=(
+                    "the name(s) or YellowDog ID(s) of the held (paused) work requirement(s) to be"
+                    " started"
+                ),
+            )
+
+        if "hold" in sys.argv[0]:
+            parser.add_argument(
+                "work_requirements",
+                nargs="*",
+                default="",
+                metavar="<work-requirement-name-or-ID>",
+                type=str,
+                help=(
+                    "the name(s) or YellowDog ID(s) of the work requirement(s) to be"
+                    " held (paused)"
+                ),
             )
 
         if "delete" in sys.argv[0]:
@@ -587,7 +692,7 @@ class CLIParser:
                 "resource_specifications",
                 nargs="+",
                 default=[],
-                metavar="<resource_specification>",
+                metavar="<resource-specification>",
                 type=str,
                 help=(
                     "the resource specifications to process (or resource IDs if used"
@@ -614,7 +719,154 @@ class CLIParser:
                 "--ids",
                 action="store_true",
                 required=False,
-                help="remove resources using their YellowDog IDs (ydids)",
+                help="remove resources using their YellowDog IDs (YDIDs)",
+            )
+
+        if "follow" in sys.argv[0]:
+            parser.add_argument(
+                "yellowdog_ids",
+                nargs="+",
+                default=[],
+                metavar="<yellowdog-id>",
+                type=str,
+                help="the YellowDog ID(s) of the item(s) to follow",
+            )
+
+        if any(
+            module in sys.argv[0]
+            for module in ["upload", "submit", "provision", "instantiate"]
+        ):
+            parser.add_argument(
+                "--content-path",
+                "-F",
+                type=str,
+                required=False,
+                help=(
+                    "the directory in which files for upload (or for user data) are"
+                    " found"
+                ),
+                metavar="<directory>",
+            )
+
+        if any(module in sys.argv[0] for module in ["provision", "instantiate"]):
+            parser.add_argument(
+                "--follow",
+                "-f",
+                action="store_true",
+                required=False,
+                help="follow progress after provisioning",
+            )
+
+        if any(module in sys.argv[0] for module in ["resize"]):
+            parser.add_argument(
+                "--follow",
+                "-f",
+                action="store_true",
+                required=False,
+                help="follow progress after resizing",
+            )
+
+        if any(
+            module in sys.argv[0]
+            for module in ["follow", "shutdown", "provision", "resize"]
+        ):
+            parser.add_argument(
+                "--auto-follow-compute-requirements",
+                "-a",
+                action="store_true",
+                required=False,
+                help=(
+                    "automatically follow the associated compute requirements when"
+                    " following worker pools"
+                ),
+            )
+        if any(
+            module in sys.argv[0]
+            for module in [
+                "follow",
+                "provision",
+                "instantiate",
+                "resize",
+                "shutdown",
+                "terminate",
+                "submit",
+                "cancel",
+                "start",
+                "hold",
+            ]
+        ):
+            parser.add_argument(
+                "--raw-events",
+                action="store_true",
+                required=False,
+                help="print the raw JSON event stream when following events",
+            )
+
+        if "cloudwizard" in sys.argv[0]:
+            parser.add_argument(
+                "operation",
+                metavar="'setup', 'teardown', 'add-ssh' or 'remove-ssh'",
+                type=str,
+                choices=["setup", "teardown", "add-ssh", "remove-ssh"],
+                help=(
+                    "the cloud wizard operation to perform: 'setup', 'teardown',"
+                    " 'add-ssh' or 'remove-ssh'"
+                ),
+            )
+            parser.add_argument(
+                "--cloud-provider",
+                required=True,
+                metavar="<name of cloud provider>",
+                type=str,
+                help=(
+                    "the name of the cloud provider (AWS, GCP, and Azure are currently"
+                    " supported)"
+                ),
+            )
+            parser.add_argument(
+                "--credentials-file",
+                required=False,
+                metavar="<file-containing-google-credentials>",
+                type=str,
+                help=(
+                    "the name of the file containing the cloud GCP credentials when"
+                    " using Cloud Wizard with GCP"
+                ),
+            )
+            parser.add_argument(
+                "--region-name",
+                "-R",
+                required=False,
+                metavar="<cloud-provider-region-name>",
+                type=str,
+                help="specify the cloud provider region name for certain operations",
+            )
+            parser.add_argument(
+                "--instance-type",
+                "-I",
+                required=False,
+                metavar="<instance-type>",
+                type=str,
+                help=(
+                    "the instance type to use in the automatically generated YellowDog"
+                    " compute requirement templates"
+                ),
+            )
+            parser.add_argument(
+                "--show-secrets",
+                action="store_true",
+                required=False,
+                help="print AWS secret key during setup",
+            )
+
+        if "abort" in sys.argv[0]:
+            parser.add_argument(
+                "task_id_list",
+                nargs="*",
+                default="",
+                metavar="<task-id>",
+                type=str,
+                help="the YellowDog ID(s) of the task(s) to abort",
             )
 
         self.args = parser.parse_args()
@@ -623,77 +875,78 @@ class CLIParser:
             docs()
             exit(0)
 
-        # Temporary notification message while we figure out the problem
-        # with the use of concurrent futures in Python 3.10+
-        if (
-            any(module in sys.argv[0] for module in ["submit"])
-            and self.args.follow
-            and sys.version_info >= (3, 10)
-        ):
-            print(
-                datetime.now().strftime("%Y-%m-%d %H:%M:%S"),
-                ":",
-                "Note: the '--follow' ('-f') option is partially supported "
-                "for Python versions 3.10 and above",
-            )
-
     @property
+    @allow_missing_attribute
     def config_file(self) -> Optional[str]:
         return self.args.config
 
     @property
+    @allow_missing_attribute
     def key(self) -> Optional[str]:
         return self.args.key
 
     @property
+    @allow_missing_attribute
     def secret(self) -> Optional[str]:
         return self.args.secret
 
     @property
+    @allow_missing_attribute
     def namespace(self) -> Optional[str]:
         return self.args.namespace
 
     @property
+    @allow_missing_attribute
     def tag(self) -> Optional[str]:
         return self.args.tag
 
     @property
+    @allow_missing_attribute
     def url(self) -> Optional[str]:
         return self.args.url
 
     @property
+    @allow_missing_attribute
     def variables(self) -> Optional[List[str]]:
         return self.args.variable
 
     @property
-    def quiet(self) -> bool:
+    @allow_missing_attribute
+    def quiet(self) -> Optional[bool]:
         return self.args.quiet
 
     @property
+    @allow_missing_attribute
     def work_req_file(self) -> Optional[str]:
         return self.args.work_requirement
 
     @property
+    @allow_missing_attribute
     def executable(self) -> Optional[str]:
         return self.args.executable
 
     @property
+    @allow_missing_attribute
     def task_type(self) -> Optional[str]:
         return self.args.task_type
 
     @property
+    @allow_missing_attribute
     def worker_pool_file(self) -> Optional[str]:
         return self.args.worker_pool
 
     @property
+    @allow_missing_attribute
     def follow(self) -> Optional[bool]:
         return self.args.follow
 
     @property
+    @allow_missing_attribute
     def abort(self) -> Optional[bool]:
         return self.args.abort
 
     @property
+    @allow_missing_attribute
     def interactive(self) -> Optional[bool]:
         return self.args.interactive
 
@@ -702,200 +955,311 @@ class CLIParser:
         self.args.interactive = interactive
 
     @property
+    @allow_missing_attribute
     def yes(self) -> Optional[bool]:
-        # This attribute is used in 'submit' without the '--yes' option
-        # being enabled for that module. Ensure that 'None' is returned.
-        try:
-            return self.args.yes
-        except AttributeError:
-            return None
+        return self.args.yes
 
     @property
+    @allow_missing_attribute
     def object_paths(self) -> Optional[bool]:
         return self.args.object_paths
 
     @property
+    @allow_missing_attribute
     def work_requirements(self) -> Optional[bool]:
         return self.args.work_requirements
 
     @property
+    @allow_missing_attribute
     def task_groups(self) -> Optional[bool]:
         return self.args.task_groups
 
     @property
+    @allow_missing_attribute
     def tasks(self) -> Optional[bool]:
         return self.args.tasks
 
     @property
+    @allow_missing_attribute
     def worker_pools(self) -> Optional[bool]:
         return self.args.worker_pools
 
     @property
+    @allow_missing_attribute
     def compute_requirements(self) -> Optional[bool]:
         return self.args.compute_requirements
 
     @property
-    def live_only(self) -> Optional[bool]:
-        return self.args.live_only
+    @allow_missing_attribute
+    def active_only(self) -> Optional[bool]:
+        return self.args.active_only
 
     @property
+    @allow_missing_attribute
     def all(self) -> Optional[bool]:
         return self.args.all
 
     @property
+    @allow_missing_attribute
     def details(self) -> Optional[bool]:
         return self.args.details
 
     @property
+    @allow_missing_attribute
     def debug(self) -> Optional[bool]:
         return self.args.debug
 
     @property
+    @allow_missing_attribute
     def use_pac(self) -> Optional[bool]:
         return self.args.pac
 
     @property
-    def directory(self) -> str:
-        return "" if self.args.directory is None else self.args.directory
+    @allow_missing_attribute
+    def no_format(self) -> Optional[bool]:
+        return self.args.no_format
 
     @property
+    @allow_missing_attribute
+    def directory(self) -> Optional[str]:
+        return self.args.directory
+
+    @property
+    @allow_missing_attribute
     def files(self) -> List[str]:
         return self.args.filenames
 
     @property
+    @allow_missing_attribute
     def flatten(self) -> Optional[bool]:
         return self.args.flatten_upload_paths
 
     @property
+    @allow_missing_attribute
     def recursive(self) -> Optional[bool]:
         return self.args.recursive
 
     @property
+    @allow_missing_attribute
     def batch(self) -> Optional[bool]:
         return self.args.batch
 
     @property
+    @allow_missing_attribute
     def dry_run(self) -> Optional[bool]:
         return self.args.dry_run
 
     @property
+    @allow_missing_attribute
     def json_raw(self) -> Optional[str]:
         return self.args.json_raw
 
     @property
+    @allow_missing_attribute
     def compute_requirement(self) -> Optional[str]:
         return self.args.compute_requirement
 
     @property
+    @allow_missing_attribute
     def task_count(self) -> Optional[int]:
         return self.args.task_count
 
     @property
+    @allow_missing_attribute
     def csv_files(self) -> Optional[List[str]]:
         return self.args.csv_file
 
     @property
+    @allow_missing_attribute
     def process_csv_only(self) -> Optional[bool]:
         return self.args.process_csv_only
 
     @property
+    @allow_missing_attribute
     def wr_ids(self) -> Optional[List[str]]:
         return self.args.work_requirement_id
 
     @property
+    @allow_missing_attribute
     def task_batch_size(self) -> Optional[int]:
         return self.args.task_batch_size
 
     @property
+    @allow_missing_attribute
     def content_path(self) -> Optional[str]:
         return self.args.content_path
 
     @property
+    @allow_missing_attribute
     def report(self) -> Optional[bool]:
         return self.args.report
 
     @property
+    @allow_missing_attribute
     def jsonnet_dry_run(self) -> Optional[bool]:
         return self.args.jsonnet_dry_run
 
     @property
-    def pause_between_batches(self) -> Optional[bool]:
+    @allow_missing_attribute
+    def pause_between_batches(self) -> Optional[int]:
         return self.args.pause_between_batches
 
     @property
-    def worker_pool_name(self) -> str:
+    @allow_missing_attribute
+    def worker_pool_name(self) -> Optional[str]:
         return self.args.worker_pool
 
     @property
-    def compute_requirement_name(self) -> str:
-        return self.args.compute_requirement
+    @allow_missing_attribute
+    def worker_pool_list(self) -> Optional[List[str]]:
+        return self.args.worker_pool_list
 
     @property
-    def work_requirement_name(self) -> str:
-        return self.args.work_requirement
+    @allow_missing_attribute
+    def compute_requirement_names(self) -> Optional[str]:
+        return self.args.compute_reqs
 
     @property
-    def object_paths_to_delete(self) -> List[str]:
+    @allow_missing_attribute
+    def work_requirement_names(self) -> Optional[str]:
+        return self.args.work_requirements
+
+    @property
+    @allow_missing_attribute
+    def object_paths_to_delete(self) -> Optional[List[str]]:
         return self.args.object_paths_to_delete
 
     @property
-    def object_paths_to_download(self) -> List[str]:
+    @allow_missing_attribute
+    def object_paths_to_download(self) -> Optional[List[str]]:
         return self.args.object_paths_to_download
 
     @property
-    def worker_pool_size(self) -> int:
+    @allow_missing_attribute
+    def worker_pool_size(self) -> Optional[int]:
         return self.args.worker_pool_size
 
     @property
+    @allow_missing_attribute
     def compute_req_resize(self) -> Optional[bool]:
         return self.args.compute_requirement
 
     @property
+    @allow_missing_attribute
     def flatten_download_paths(self) -> Optional[bool]:
         return self.args.flatten
 
     @property
+    @allow_missing_attribute
     def compute_templates(self) -> Optional[bool]:
         return self.args.compute_templates
 
     @property
+    @allow_missing_attribute
     def source_templates(self) -> Optional[bool]:
         return self.args.source_templates
 
     @property
+    @allow_missing_attribute
     def keyrings(self) -> Optional[bool]:
         return self.args.keyrings
 
     @property
+    @allow_missing_attribute
     def image_families(self) -> Optional[bool]:
         return self.args.image_families
 
     @property
-    def namespaces(self) -> Optional[bool]:
-        return self.args.namespaces
+    @allow_missing_attribute
+    def namespace_storage_configurations(self) -> Optional[bool]:
+        return self.args.namespace_storage_configurations
 
     @property
+    @allow_missing_attribute
     def instances(self) -> Optional[bool]:
         return self.args.instances
 
     @property
+    @allow_missing_attribute
     def show_keyring_passwords(self) -> Optional[bool]:
         return self.args.show_keyring_passwords
 
     @property
+    @allow_missing_attribute
     def ids(self) -> Optional[bool]:
         return self.args.ids
 
     @property
-    def resource_specifications(self) -> List[str]:
+    @allow_missing_attribute
+    def resource_specifications(self) -> Optional[List[str]]:
         return self.args.resource_specifications
 
+    @property
+    @allow_missing_attribute
+    def yellowdog_ids(self) -> Optional[List[str]]:
+        return self.args.yellowdog_ids
+
+    @property
+    @allow_missing_attribute
+    def auto_cr(self) -> Optional[bool]:
+        return self.args.auto_follow_compute_requirements
+
+    @property
+    @allow_missing_attribute
+    def raw_events(self) -> Optional[bool]:
+        return self.args.raw_events
+
+    @property
+    @allow_missing_attribute
+    def reverse(self) -> Optional[bool]:
+        return self.args.reverse
+
+    @property
+    @allow_missing_attribute
+    def operation(self) -> Optional[str]:
+        return self.args.operation
+
+    @property
+    @allow_missing_attribute
+    def cloud_provider(self) -> Optional[str]:
+        return self.args.cloud_provider
+
+    @property
+    @allow_missing_attribute
+    def credentials_file(self) -> Optional[str]:
+        return self.args.credentials_file
+
+    @property
+    @allow_missing_attribute
+    def region_name(self) -> Optional[str]:
+        return self.args.region_name
+
+    @property
+    @allow_missing_attribute
+    def instance_type(self) -> Optional[str]:
+        return self.args.instance_type
+
+    @property
+    @allow_missing_attribute
+    def show_secrets(self) -> Optional[bool]:
+        return self.args.show_secrets
+
+    @property
+    @allow_missing_attribute
+    def hold(self) -> Optional[bool]:
+        return self.args.hold
+
+    @property
+    @allow_missing_attribute
+    def task_id_list(self) -> Optional[List[str]]:
+        return self.args.task_id_list
+
 
 def lookup_module_description(module_name: str) -> Optional[str]:
     """
     Descriptive string for the module's purpose.
     """
-    prefix = "YellowDog example utility for "
+    prefix = "YellowDog command line utility for "
     suffix = None
 
     if "submit" in module_name:
@@ -907,7 +1271,7 @@ def lookup_module_description(module_name: str) -> Optional[str]:
     elif "cancel" in module_name:
         suffix = "cancelling Work Requirements"
     elif "create" in module_name:
-        suffix = "creating/updating resources"
+        suffix = "creating and updating resources"
     elif "download" in module_name:
         suffix = "downloading objects from the Object Store"
     elif "delete" in module_name:
@@ -917,15 +1281,23 @@ def lookup_module_description(module_name: str) -> Optional[str]:
     elif "terminate" in module_name:
         suffix = "terminating Compute Requirements"
     elif "list" in module_name:
-        suffix = "listing entities"
+        suffix = "listing all kinds of YellowDog items"
     elif "instantiate" in module_name:
-        suffix = "provisioning a Compute Requirement"
+        suffix = "instantiating a Compute Requirement"
     elif "upload" in module_name:
         suffix = "uploading objects to the Object Store"
     elif "remove" in module_name:
         suffix = "removing resources"
     elif "resize" in module_name:
-        suffix = "resizing Worker Pools"
+        suffix = "resizing Worker Pools and Compute Requirements"
+    elif "follow" in module_name:
+        suffix = "following event streams"
+    elif "cloudwizard" in module_name:
+        suffix = "setting up cloud accounts and YellowDog resources"
+    elif "start" in module_name:
+        suffix = "starting held (paused) Work Requirements"
+    elif "hold" in module_name:
+        suffix = "holding (pausing) running Work Requirements"
 
     return None if suffix is None else prefix + suffix
 
