@@ -87,7 +87,7 @@ def load_config_common() -> ConfigCommon:
         common_section = CONFIG_TOML.get(COMMON_SECTION, {})
 
         # Check for IMPORT directive (common section in a separate file)
-        common_section_import_file = common_section.get(IMPORT, None)
+        common_section_import_file = common_section.get(IMPORT_COMMON, None)
         if common_section_import_file is not None:
             common_section = import_toml(common_section_import_file)
 
@@ -233,9 +233,10 @@ def load_config_work_requirement() -> Optional[ConfigWorkRequirement]:
         )
 
         return ConfigWorkRequirement(
+            add_yd_env_vars=wr_section.get(ADD_YD_ENV_VARS, False),
             always_upload=wr_section.get(ALWAYS_UPLOAD, True),
             args=wr_section.get(ARGS, []),
-            capture_taskoutput=wr_section.get(CAPTURE_TASKOUTPUT, False),
+            upload_taskoutput=wr_section.get(UPLOAD_TASKOUTPUT, False),
             completed_task_ttl=wr_section.get(COMPLETED_TASK_TTL, None),
             csv_files=csv_files,
             docker_env=wr_section.get(DOCKER_ENV, None),
@@ -281,6 +282,7 @@ def load_config_work_requirement() -> Optional[ConfigWorkRequirement]:
             worker_tags=worker_tags,
             wr_data_file=wr_data_file,
             wr_name=wr_section.get(WR_NAME, None),
+            wr_tag=wr_section.get(WR_TAG, None),
         )
     except KeyError as e:
         print_error(f"Missing configuration data: {e}")
@@ -349,6 +351,7 @@ def load_config_worker_pool() -> Optional[ConfigWorkerPool]:
                 COMPUTE_REQUIREMENT_BATCH_SIZE, CR_BATCH_SIZE_DEFAULT
             ),
             compute_requirement_data_file=compute_requirement_data_file,
+            cr_tag=wp_section.get(CR_TAG, None),
             idle_node_shutdown_timeout=wp_section.get(IDLE_NODE_SHUTDOWN_TIMEOUT, 5.0),
             idle_pool_shutdown_timeout=wp_section.get(IDLE_POOL_SHUTDOWN_TIMEOUT, 30.0),
             images_id=wp_section.get(IMAGES_ID, None),
