@@ -7,7 +7,8 @@ A script to Provision a Worker Pool.
 from dataclasses import dataclass
 from datetime import timedelta
 from math import ceil, floor
-from typing import Dict, List, Optional
+from os.path import dirname
+from typing import Dict, List
 
 import requests
 from yellowdog_client.common.iso_datetime import iso_timedelta_format
@@ -18,15 +19,17 @@ from yellowdog_client.model import (
     ProvisionedWorkerPoolProperties,
 )
 
-from yd_commands.args import ARGS_PARSER
-from yd_commands.config import (
-    WP_VARIABLES_PREFIX,
-    ConfigWorkerPool,
-    generate_id,
-    link_entity,
-    load_config_worker_pool,
+from yd_commands.config_types import ConfigWorkerPool
+from yd_commands.follow_utils import follow_ids
+from yd_commands.id_utils import YDIDType, get_ydid_type
+from yd_commands.load_config import CONFIG_FILE_DIR, load_config_worker_pool
+from yd_commands.printing import (
+    print_error,
+    print_log,
+    print_worker_pool,
+    print_yd_object,
 )
-from yd_commands.config_keys import (
+from yd_commands.property_names import (
     IMAGES_ID,
     INSTANCE_TAGS,
     MAINTAIN_INSTANCE_COUNT,
@@ -38,18 +41,19 @@ from yd_commands.config_keys import (
     USERDATA,
     WORKER_TAG,
 )
-from yd_commands.printing import (
-    print_error,
-    print_log,
-    print_worker_pool,
-    print_yd_object,
+from yd_commands.provision_utils import (
+    get_image_family_id,
+    get_template_id,
+    get_user_data_property,
 )
-from yd_commands.provision_utils import get_user_data_property
+from yd_commands.settings import WP_VARIABLES_POSTFIX, WP_VARIABLES_PREFIX
+from yd_commands.utils import add_batch_number_postfix, generate_id, link_entity
 from yd_commands.variables import (
     load_json_file_with_variable_substitutions,
     load_jsonnet_file_with_variable_substitutions,
+    resolve_filename,
 )
-from yd_commands.wrapper import CLIENT, CONFIG_COMMON, main_wrapper
+from yd_commands.wrapper import ARGS_PARSER, CLIENT, CONFIG_COMMON, main_wrapper
 
 
 # Specifies the cardinality for a Worker Pool batch
@@ -61,6 +65,7 @@ class WPBatch:
 
 
 CONFIG_WP: ConfigWorkerPool = load_config_worker_pool()
+GENERATED_ID = generate_id("wp" + "_" + CONFIG_COMMON.name_tag)
 
 
 @main_wrapper
@@ -70,13 +75,23 @@ def main():
         if ARGS_PARSER.worker_pool_file is None
         else ARGS_PARSER.worker_pool_file
     )
+
+    # Where do we find the data files?
+    # content-path > wp_json_file location > config file location
+    files_directory = (
+        (CONFIG_FILE_DIR if wp_json_file is None else dirname(wp_json_file))
+        if ARGS_PARSER.content_path is None
+        else ARGS_PARSER.content_path
+    )
+
     if wp_json_file is not None:
+        wp_json_file = resolve_filename(files_directory, wp_json_file)
         print_log(f"Loading Worker Pool data from: '{wp_json_file}'")
         create_worker_pool_from_json(wp_json_file)
     elif CONFIG_WP.template_id is None:
         print_error("No template_id supplied")
     else:
-        create_worker_pool()
+        create_worker_pool_from_toml()
 
 
 def create_worker_pool_from_json(wp_json_file: str) -> None:
@@ -85,11 +100,11 @@ def create_worker_pool_from_json(wp_json_file: str) -> None:
     """
     if wp_json_file.lower().endswith(".jsonnet"):
         wp_data = load_jsonnet_file_with_variable_substitutions(
-            wp_json_file, prefix=WP_VARIABLES_PREFIX
+            wp_json_file, prefix=WP_VARIABLES_PREFIX, postfix=WP_VARIABLES_POSTFIX
         )
     else:
         wp_data = load_json_file_with_variable_substitutions(
-            wp_json_file, prefix=WP_VARIABLES_PREFIX
+            wp_json_file, prefix=WP_VARIABLES_PREFIX, postfix=WP_VARIABLES_POSTFIX
         )
 
     # Some values are configurable via the TOML configuration file;
@@ -101,16 +116,19 @@ def create_worker_pool_from_json(wp_json_file: str) -> None:
             # Generate a default name
             (
                 "requirementName",
-                (
-                    CONFIG_WP.name
-                    if CONFIG_WP.name is not None
-                    else generate_id("wp" + "_" + CONFIG_COMMON.name_tag)
-                ),
+                (CONFIG_WP.name if CONFIG_WP.name is not None else GENERATED_ID),
             ),
             ("requirementNamespace", CONFIG_COMMON.namespace),
-            ("requirementTag", CONFIG_COMMON.name_tag),
+            (
+                "requirementTag",
+                (
+                    CONFIG_COMMON.name_tag
+                    if CONFIG_WP.cr_tag is None
+                    else CONFIG_WP.cr_tag
+                ),
+            ),
             (TEMPLATE_ID, CONFIG_WP.template_id),
-            (USERDATA, get_user_data_property(CONFIG_WP)),
+            (USERDATA, get_user_data_property(CONFIG_WP, ARGS_PARSER.content_path)),
             (IMAGES_ID, CONFIG_WP.images_id),
             (INSTANCE_TAGS, CONFIG_WP.instance_tags),
         ]:
@@ -129,6 +147,17 @@ def create_worker_pool_from_json(wp_json_file: str) -> None:
             )
             reqt_template_usage[TARGET_INSTANCE_COUNT] = CONFIG_WP.target_instance_count
 
+        # Allow a Compute Requirement Template name to be used instead of ID
+        reqt_template_usage[TEMPLATE_ID] = get_template_id(
+            CLIENT, reqt_template_usage[TEMPLATE_ID]
+        )
+
+        # Allow Image Family name to be used instead of ID
+        if reqt_template_usage.get(IMAGES_ID) is not None:
+            reqt_template_usage[IMAGES_ID] = get_image_family_id(
+                CLIENT, reqt_template_usage[IMAGES_ID]
+            )
+
         # provisionedProperties insertions
         provisioned_properties = wp_data["provisionedProperties"]
 
@@ -147,7 +176,7 @@ def create_worker_pool_from_json(wp_json_file: str) -> None:
                             timedelta(minutes=CONFIG_WP.idle_node_shutdown_timeout)
                         ),
                     }
-                    if CONFIG_WP.idle_node_shutdown_enabled
+                    if CONFIG_WP.idle_node_shutdown_timeout != 0
                     else {"enabled": False}
                 ),
             ),
@@ -160,7 +189,7 @@ def create_worker_pool_from_json(wp_json_file: str) -> None:
                             timedelta(minutes=CONFIG_WP.idle_pool_shutdown_timeout)
                         ),
                     }
-                    if CONFIG_WP.idle_pool_shutdown_enabled
+                    if CONFIG_WP.idle_pool_shutdown_timeout != 0
                     else {"enabled": False}
                 ),
             ),
@@ -184,10 +213,14 @@ def create_worker_pool_from_json(wp_json_file: str) -> None:
     except KeyError as e:
         raise Exception(f"Key error in JSON Worker Pool definition: {e}")
 
+    template_id = wp_data["requirementTemplateUsage"]["templateId"]
+    if get_ydid_type(template_id) != YDIDType.CR_TEMPLATE:
+        raise Exception(f"Not a valid Compute Requirement Template ID: '{template_id}'")
+
     if ARGS_PARSER.dry_run:
         print_log("Dry-run: Printing JSON Worker Pool specification")
         print_yd_object(wp_data)
-        print_log("Dry run: Complete")
+        print_log("Dry-run: Complete")
         return
 
     response = requests.post(
@@ -201,15 +234,21 @@ def create_worker_pool_from_json(wp_json_file: str) -> None:
         print_log(f"Provisioned Worker Pool '{name}' ({id})")
         if ARGS_PARSER.quiet:
             print(id)
+        if ARGS_PARSER.follow:
+            print_log("Following Worker Pool event stream")
+            follow_ids([id], auto_cr=ARGS_PARSER.auto_cr)
     else:
         print_error(f"Failed to provision Worker Pool '{name}'")
         raise Exception(f"{response.text}")
 
 
-def create_worker_pool():
+def create_worker_pool_from_toml():
     """
     Create the Worker Pool
     """
+
+    global CONFIG_WP
+
     # Check for well-configured node quantities
     if not (
         CONFIG_WP.min_nodes <= CONFIG_WP.target_instance_count <= CONFIG_WP.max_nodes
@@ -221,6 +260,23 @@ def create_worker_pool():
         )
         raise Exception("Malformed configuration")
 
+    # Allow the Compute Requirement Template name to be used instead of ID
+    CONFIG_WP.template_id = get_template_id(
+        client=CLIENT, template_id_or_name=CONFIG_WP.template_id
+    )
+
+    if get_ydid_type(CONFIG_WP.template_id) != YDIDType.CR_TEMPLATE:
+        raise Exception(
+            "Not a valid Compute Requirement Template ID or name:"
+            f" '{CONFIG_WP.template_id}'"
+        )
+
+    # Allow the Image Family name to be used instead of ID
+    if CONFIG_WP.images_id is not None:
+        CONFIG_WP.images_id = get_image_family_id(
+            client=CLIENT, image_family_id_or_name=CONFIG_WP.images_id
+        )
+
     node_boot_timeout = (
         None
         if CONFIG_WP.node_boot_timeout is None
@@ -232,7 +288,7 @@ def create_worker_pool():
             enabled=True,
             timeout=timedelta(minutes=CONFIG_WP.idle_node_shutdown_timeout),
         )
-        if CONFIG_WP.idle_node_shutdown_enabled
+        if CONFIG_WP.idle_node_shutdown_timeout != 0
         else AutoShutdown(enabled=False)
     )
 
@@ -241,7 +297,7 @@ def create_worker_pool():
             enabled=True,
             timeout=timedelta(minutes=CONFIG_WP.idle_pool_shutdown_timeout),
         )
-        if CONFIG_WP.idle_pool_shutdown_enabled
+        if CONFIG_WP.idle_pool_shutdown_timeout != 0
         else AutoShutdown(enabled=False)
     )
 
@@ -272,11 +328,16 @@ def create_worker_pool():
         CONFIG_WP.max_nodes,
     )
     num_batches = len(batches)
+
+    worker_pool_ids: List[str] = []
     if num_batches > 1:
         print_log(f"Batching into {num_batches} Compute Requirements")
+
     for batch_number in range(num_batches):
-        id = generate_wp_batch_name(
-            name=CONFIG_WP.name, batch_number=batch_number, num_batches=num_batches
+        id = add_batch_number_postfix(
+            name=(CONFIG_WP.name if CONFIG_WP.name is not None else GENERATED_ID),
+            batch_number=batch_number,
+            num_batches=num_batches,
         )
         if num_batches > 1:
             print_log(
@@ -293,8 +354,12 @@ def create_worker_pool():
                 requirementNamespace=CONFIG_COMMON.namespace,
                 requirementName=id,
                 targetInstanceCount=batches[batch_number].initial_nodes,
-                requirementTag=CONFIG_COMMON.name_tag,
-                userData=get_user_data_property(CONFIG_WP),
+                requirementTag=(
+                    CONFIG_COMMON.name_tag
+                    if CONFIG_WP.cr_tag is None
+                    else CONFIG_WP.cr_tag
+                ),
+                userData=get_user_data_property(CONFIG_WP, ARGS_PARSER.content_path),
                 imagesId=CONFIG_WP.images_id,
                 instanceTags=CONFIG_WP.instance_tags,
                 maintainInstanceCount=False,  # Must be false for Worker Pools
@@ -314,6 +379,8 @@ def create_worker_pool():
                     provisioned_worker_pool_properties,
                 )
                 print_log(f"Created {link_entity(CONFIG_COMMON.url, worker_pool)}")
+                print_log(f"YellowDog ID is '{worker_pool.id}'")
+                worker_pool_ids.append(worker_pool.id)
                 if ARGS_PARSER.quiet:
                     print(worker_pool.id)
             else:
@@ -323,12 +390,11 @@ def create_worker_pool():
                 )
 
         except Exception as e:
-            print_error(f"Unable to provision worker pool")
-            raise Exception(e)
+            raise Exception(f"Unable to provision worker pool: {e}")
 
     idle_node_shutdown_string = (
         f"time limit is {CONFIG_WP.idle_node_shutdown_timeout} minute(s)"
-        if CONFIG_WP.idle_node_shutdown_enabled
+        if CONFIG_WP.idle_node_shutdown_timeout != 0
         else "is **disabled**"
     )
 
@@ -340,19 +406,24 @@ def create_worker_pool():
     )
 
     idle_pool_shutdown = (
-        "enabled" if CONFIG_WP.idle_pool_shutdown_enabled else "disabled"
+        "enabled" if CONFIG_WP.idle_pool_shutdown_timeout != 0 else "disabled"
     )
     idle_pool_shutdown_msg = f"Worker Pool auto-shutdown is {idle_pool_shutdown}"
     idle_pool_shutdown_msg = (
         idle_pool_shutdown_msg
         + f" with a delay of {CONFIG_WP.idle_pool_shutdown_timeout} minute(s)"
-        if CONFIG_WP.idle_pool_shutdown_enabled
+        if CONFIG_WP.idle_pool_shutdown_timeout != 0
         else idle_pool_shutdown_msg
     )
     print_log(idle_pool_shutdown_msg)
 
     if ARGS_PARSER.dry_run:
-        print_log("Dry run: Complete")
+        print_log("Dry-run: Complete")
+        return
+
+    if ARGS_PARSER.follow:
+        print_log("Following Worker Pool event stream(s)")
+        follow_ids(worker_pool_ids, auto_cr=ARGS_PARSER.auto_cr)
 
 
 def _allocate_nodes_to_batches(
@@ -398,23 +469,6 @@ def _allocate_nodes_to_batches(
     return batches
 
 
-def generate_wp_batch_name(
-    name: Optional[str], batch_number: int, num_batches: int
-) -> str:
-    """
-    Generate the name of a Worker Pool
-    """
-
-    # Standard automatic name generation
-    if name is None:
-        return generate_id("wp" + "_" + CONFIG_COMMON.name_tag)
-
-    # Use supplied name, with counter if multiple batches
-    if num_batches > 1:
-        name += "_" + str(batch_number + 1).zfill(len(str(num_batches)))
-    return name
-
-
 # Entry point
 if __name__ == "__main__":
     main()
