@@ -278,11 +278,17 @@ def submit_work_requirement(
 
     # Create the Work Requirement
     priority = check_float_or_int(wr_data.get(PRIORITY, CONFIG_WR.priority))
+    wr_tag = check_str(
+        wr_data.get(
+            WR_TAG,
+            CONFIG_COMMON.name_tag if CONFIG_WR.wr_tag is None else CONFIG_WR.wr_tag,
+        )
+    )
     work_requirement = WorkRequirement(
         namespace=CONFIG_COMMON.namespace,
         name=ID,
         taskGroups=task_groups,
-        tag=CONFIG_COMMON.name_tag,
+        tag=wr_tag,
         priority=priority,
     )
     if not ARGS_PARSER.dry_run:
@@ -295,6 +301,9 @@ def submit_work_requirement(
             f"({work_requirement.name})"
         )
         print_log(f"YellowDog ID is '{work_requirement.id}'")
+        if ARGS_PARSER.hold:
+            CLIENT.work_client.hold_work_requirement(work_requirement)
+            print_log("Work Requirement status is set to 'HELD'")
     else:
         global WR_SNAPSHOT
         WR_SNAPSHOT.set_work_requirement(work_requirement)
@@ -387,7 +396,8 @@ def create_task_group(
             check_list(task_group_data.get(TASK_TYPES, wr_data.get(TASK_TYPES, [])))
         ).union(task_types_from_tasks)
     )
-    if config_wr.task_type is not None and config_wr.task_type not in task_types:
+    # Use the task type from the config file if present and task_types is empty
+    if config_wr.task_type is not None and len(task_types) == 0:
         task_types.append(config_wr.task_type)
     if len(task_types) == 0:
         raise Exception(f"No Task Type(s) specified in Task Group '{task_group_name}'")
@@ -496,6 +506,7 @@ def create_task_group(
             task_group_data.get(PRIORITY, wr_data.get(PRIORITY, config_wr.priority))
         ),
         completedTaskTtl=completed_task_ttl,
+        tag=task_group_data.get(TASK_GROUP_TAG, None),
     )
 
     print_log(f"Generated Task Group '{task_group_name}'")
@@ -629,6 +640,18 @@ def add_tasks_to_task_group(
                 )
             )
 
+            flatten_input_paths: Optional[FlattenPath] = None
+            if check_bool(
+                task.get(
+                    FLATTEN_PATHS,
+                    task_group_data.get(
+                        FLATTEN_PATHS,
+                        wr_data.get(FLATTEN_PATHS, config_wr.flatten_input_paths),
+                    ),
+                )
+            ):
+                flatten_input_paths = FlattenPath.FILE_NAME_ONLY
+
             # Task timeout is automatically inherited from the Task Group level
             # unless overridden by the Task
             task_timeout_minutes = check_float_or_int(
@@ -791,10 +814,10 @@ def add_tasks_to_task_group(
             # Add TaskOutput to 'outputs'?
             if check_bool(
                 task.get(
-                    CAPTURE_TASKOUTPUT,
+                    UPLOAD_TASKOUTPUT,
                     task_group_data.get(
-                        CAPTURE_TASKOUTPUT,
-                        wr_data.get(CAPTURE_TASKOUTPUT, config_wr.capture_taskoutput),
+                        UPLOAD_TASKOUTPUT,
+                        wr_data.get(UPLOAD_TASKOUTPUT, config_wr.upload_taskoutput),
                     ),
                 )
             ):
@@ -849,6 +872,7 @@ def add_tasks_to_task_group(
                     outputs=outputs,
                     task_timeout=task_timeout,
                     flatten_upload_paths=flatten_upload_paths,
+                    flatten_input_paths=flatten_input_paths,
                     add_yd_env_vars=add_yd_env_vars,
                 )
             )
@@ -878,7 +902,7 @@ def add_tasks_to_task_group(
     if not ARGS_PARSER.dry_run:
         if num_tasks > 0:
             print_log(
-                f"Added a total of {num_tasks} Task(s) to Task Group"
+                f"Added a total of {num_tasks:,d} Task(s) to Task Group"
                 f" '{task_group.name}'"
             )
         else:
@@ -1116,6 +1140,7 @@ def create_task(
     outputs: Optional[List[TaskOutput]],
     task_timeout: Optional[timedelta],
     flatten_upload_paths: bool = False,
+    flatten_input_paths: Optional[FlattenPath] = None,
     add_yd_env_vars: bool = False,
 ) -> Task:
     """
@@ -1142,30 +1167,21 @@ def create_task(
             flattenInputPaths=flatten_input_paths,
             taskData=task_data_property,
             timeout=task_timeout,
+            tag=task_tag,
         )
 
-    # Flatten paths for downloaded files?
-    flatten_input_paths: Optional[FlattenPath] = None
-    if check_bool(
-        task_data.get(
-            FLATTEN_PATHS,
-            task_group_data.get(
-                FLATTEN_PATHS,
-                wr_data.get(FLATTEN_PATHS, config_wr.flatten_input_paths),
-            ),
-        )
-    ):
-        flatten_input_paths = FlattenPath.FILE_NAME_ONLY
+    task_tag = task_data.get(TASK_TAG, None)
 
-    # Add Task details to the environment as a convenience
-    if task_type != "docker" and add_yd_env_vars:
+    # Optionally add Task details to the environment as a convenience
+    if add_yd_env_vars and task_type != "docker":
         env_copy[YD_TASK_NAME] = task_name
         env_copy[YD_TASK_NUMBER] = str(task_number)
         env_copy[YD_TASK_GROUP_NAME] = tg_name
         env_copy[YD_TASK_GROUP_NUMBER] = str(tg_number)
         env_copy[YD_WORK_REQUIREMENT_NAME] = ID
         env_copy[YD_NAMESPACE] = CONFIG_COMMON.namespace
-        env_copy[YD_TAG] = CONFIG_COMMON.name_tag
+        if task_tag is not None:
+            env_copy[YD_TAG] = task_tag
 
     # Special processing for Bash, Python & PowerShell tasks if the 'executable'
     # property is set. The script is uploaded if this hasn't already been done,
@@ -1226,16 +1242,17 @@ def create_task(
                 ),
             )
         )
+        # Optionally Task details to the container environment as a convenience
         docker_env_list = []
         if add_yd_env_vars:
-            # Add entity names to the container env. for convenience
             docker_env_list += ["--env", f"{YD_TASK_NAME}={task_name}"]
             docker_env_list += ["--env", f"{YD_TASK_NUMBER}={task_number}"]
             docker_env_list += ["--env", f"{YD_TASK_GROUP_NAME}={tg_name}"]
             docker_env_list += ["--env", f"{YD_TASK_GROUP_NUMBER}={tg_number}"]
             docker_env_list += ["--env", f"{YD_WORK_REQUIREMENT_NAME}={ID}"]
             docker_env_list += ["--env", f"{YD_NAMESPACE}={CONFIG_COMMON.namespace}"]
-            docker_env_list += ["--env", f"{YD_TAG}={CONFIG_COMMON.name_tag}"]
+            if task_tag is not None:
+                docker_env_list += ["--env", f"{YD_TAG}={task_tag}"]
 
         if docker_env is not None:
             for key, value in docker_env.items():
@@ -1347,6 +1364,10 @@ def submit_json_raw(wr_file: str):
         print_error(f"Failed to create Work Requirement '{wr_name}'")
         raise Exception(f"{response.text}")
 
+    if ARGS_PARSER.hold:
+        CLIENT.work_client.hold_work_requirement_by_id(wr_id)
+        print_log("Work Requirement status set to 'HELD'")
+
     # Submit Tasks to the Work Requirement
     # Collect 'VERIFY_AT_START' files
     verify_at_start_files = set()
